Unit VTreeDriver;

Interface

Uses
  Windows;

Type
  TDriverMajorFunctions = Packed Array [0..28] Of Pointer;

  TSnapshotType = (stDriverList, stDriverInfo, stDeviceInfo);

  TVpbSnapshot = Record
    VpbAddress : Pointer;
    Name : WideString;
    FileSystemDevice : Pointer;
    VolumeDevice : Pointer;
    Flags : Cardinal;
    ReferenceCount : Cardinal;
    end;
  PVpbSnapshot = ^TVpbSnapshot;

  TDeviceSnapshot = Record
    Name : WideString;
    Address : Pointer;
    DriverName : WideString;
    DriverAddress : Pointer;
    NumberOfLowerDevices : Integer;
    LowerDevices : Array Of Pointer;
    NumberOfUpperDevices : Integer;
    UpperDevices : Array Of Pointer;
    DisplayName : WideString;
    Description : WideString;
    Vendor : WideString;
    ClassName : WideString;
    Location : WideString;
    Enumerator : WideString;
    ClassGuid : WideString;
    Flags : Cardinal;
    Characteristics : Cardinal;
    DeviceType : Cardinal;
    DiskDeviceAddress : Pointer;
    Vpb : PVpbSnapshot;
    end;
  PDeviceSnapshot = ^TDeviceSnapshot;

  TDriverSnapshot = Record
    Name : WideString;
    Address : Pointer;
    DriverEntry : Pointer;
    DriverUnload : Pointer;
    StartIo : Pointer;
    Flags : Cardinal;
    ImageBase : Pointer;
    ImageSize : Cardinal;
    ImagePath : WideString;
    MajorFunction : TDriverMajorFunctions;
    NumberOfDevices : Integer;
    Devices : Array Of Pointer;
    end;
  PDriverSnapshot = ^TDriverSnapshot;

  TSnapshot = Record
    NumberOfDrivers : Integer;
    Drivers : Array Of Pointer;
    end;
  PSnapshot = ^TSnapshot;

  SNAPSHOT_DRIVERLIST = Record
    Size : NativeInt;
    NumberOfDrivers : NativeInt;
    DriversOffset : NativeInt;
    end;
  PSNAPSHOT_DRIVERLIST = ^SNAPSHOT_DRIVERLIST;

  SNAPSHOT_DRIVERINFO = Record
    Size : NativeInt;
    ImageBase : Pointer;
    ImageSize : Cardinal;
    Flags : Cardinal;
    StartIo : Pointer;
    DriverEntry : Pointer;
    DriverUnload : Pointer;
    NameOffset : NativeInt;
    ObjectAddress : Pointer;
    NumberOfDevices : NativeInt;
    DevicesOffset : NativeInt;
    MajorFunction : TDriverMajorFunctions;
    end;
  PSNAPSHOT_DRIVERINFO = ^SNAPSHOT_DRIVERINFO;

  SNAPSHOT_VPBINFO = Record
    Size : ULONG;
    Flags : ULONG;
    SerialNumber : ULONG;
    ReferenceCount : ULONG;
    FileSystemDevice : Pointer;
    VolumeDevice : Pointer;
    VolumeLabel : NativeUInt;
    end;
  PSNAPSHOT_VPBINFO = ^SNAPSHOT_VPBINFO;

  SNAPSHOT_DEVICEINFO = Record
    Size : NativeInt;
    NameOffset : NativeInt;
    ObjectAddress : Pointer;
    Flags : ULONG;
    Characteristics : ULONG;
    DeviceType : ULONG;
    NumberOfLowerDevices : NativeInt;
    LowerDevicesOffset : NativeInt;
    NumberOfUpperDevices : NativeInt;
    UpperDevicesOffset : NativeInt;
    DisplayNameOffset : NativeInt;
    VendorNameOffset : NativeInt;
    DescriptionOffset : NativeInt;
    EnumeratorOffset : NativeInt;
    LocationOffset : NativeInt;
    ClassNameOffset : NativeInt;
    ClassGuidOffset : NativeInt;
    DiskDevice : Pointer;
    Vpb : Pointer;
    VpbSnapshot : Pointer;
    end;
  PSNAPSHOT_DEVICEINFO = ^SNAPSHOT_DEVICEINFO;

Function DriverInstall:Boolean;
Procedure DriverUninstall;
Function DriverLoad:Boolean;
Procedure DriverUnload;

Function DriverCreateSnapshot(Var ASnapshot:Pointer):Boolean;
Function DriverFreeSnapshot(ASnapshot:Pointer):Boolean;

Implementation

Uses
  SysUtils, scmDrivers, Utils;

Const
  DriverServiceName = 'VrtuleTree';
  DriverFIleName = 'VrtuleTree.sys';
  DriverDeviceName = '\\.\VrtuleTree';

  IOCTL_VRTULETREE_CREATE_SNAPSHOT          = $228008;


Var
  hDevice : THandle;

Function DriverInstall:Boolean;
Var
  FullFileName : WideString;
begin
FullFileName := Format('%s%s', [ExtractFilePath(ParamStr(0)), DriverFileName]);
Result := ScmDriverInstall(DriverServiceName, FullFileName);
end;

Procedure DriverUninstall;
begin
ScmDriverUninstall(DriverServiceName);
end;

Function DriverLoad:Boolean;
begin
Result := ScmDriverLoad(DriverServiceName);
If Result Then
  begin
  hDevice := CreateFileW(DriverDeviceName, GENERIC_ALL, 0, Nil, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
  Result := hDevice <> INVALID_HANDLE_VALUE;
  If Not Result Then
    ScmDriverUnload(DriverServiceName);
  end;
end;

Procedure DriverUnload;
begin
CloseHandle(hDevice);
ScmDriverUnload(DriverServiceName);
end;

Function DriverIOCTL(ACode:Cardinal; AInBuffer:Pointer; AInBufferLength:Integer; AOutBUffer:Pointer; AOutBufferLength:Cardinal):Boolean;
Var
  Dummy : DWORD;
begin
Result := DeviceIoControl(hDevice, ACode, AInBuffer, AInBufferLength, AOutBUffer, AOutBufferLength, Dummy, Nil);
end;

Function DriverCreateSnapshot(Var ASnapshot:Pointer):Boolean;
begin
Result := DriverIOCTL(IOCTL_VRTULETREE_CREATE_SNAPSHOT, Nil, 0, @ASnapshot, SizeOf(ASnapshot));
end;

Function DriverFreeSnapshot(ASnapshot:Pointer):Boolean;
Var
  I, J : Integer;
  DriverList : PSNAPSHOT_DRIVERLIST;
  DriverInfo : PSNAPSHOT_DRIVERINFO;
  DeviceInfo : PSNAPSHOT_DEVICEINFO;
  DriverInfoHandle : PPointer;
  DeviceInfoHandle : PPointer;
begin
DriverList := ASnapshot;
DriverInfoHandle := PPointer(PByte(DriverList) + DriverList.DriversOffset);
For I := 0 To DriverList.NumberOfDrivers - 1 Do
  begin
  DriverInfo := DriverInfoHandle^;
  DeviceInfoHandle := PPointer(PByte(DriverInfo) + DriverInfo.DevicesOffset);
  For J := 0 To DriverInfo.NumberOfDevices - 1 Do
    begin
    DeviceInfo := DeviceInfoHandle^;
    If Assigned(DeviceInfo.VpbSnapshot) Then
      VirtualFree(DeviceInfo.VpbSnapshot, 0, MEM_RELEASE);

    VirtualFree(DeviceInfo, 0, MEM_RELEASE);
    Inc(DeviceInfoHandle);
    end;

  VirtualFree(DriverInfo, 0, MEM_RELEASE);
  Inc(DriverInfoHandle);
  end;

VirtualFree(DriverList, 0, MEM_RELEASE);
end;


End.

